@@ -329,6 +329,7 @@ sub compile_file_in {
 BEGIN {
   my %escape = (qw(< &lt;
 		   > &gt;
+		   --> --&gt;
 		   " &quot;
 		   & &amp;)
 		, "\'", "&#39;");
@@ -349,9 +350,9 @@ BEGIN {
 	} elsif (ref $str eq 'SCALAR') {
 	  # PASS Thru. (Already escaped)
 	  $$str;
-	} elsif (my $sub = UNIVERSAL::can($str, 'as_escaped')) {
-	  $sub->($str);
-	} elsif ($sub = UNIVERSAL::can($str, 'cf_pairs')) {
+	} elsif (_is_escapable($str)) {
+	  $str->as_escaped;
+	} elsif (my $sub = UNIVERSAL::can($str, 'cf_pairs')) {
 	  ref($str).'->new('.(join(", ", map {
 	    my ($k, $v) = @$_;
 	    "$k => " . do {
@@ -369,7 +370,7 @@ BEGIN {
 	  # XXX: Is this secure???
 	  # XXX: Should be JSON?
 	  my $copy = terse_dump($str);
-	  $copy =~ s{([<\"])}{$escape{$1}}g; # XXX: Minimum. May be insecure.
+	  $copy =~ s{([<\"]|-->)}{$escape{$1}}g; # XXX: Minimum. May be insecure.
 	  $copy;
 	}
       };
@@ -378,9 +379,21 @@ BEGIN {
   }
 }
 
+# XXX: Since method name "as_escaped" conflicts with CGen::Perl->as_escaped,
+# We need a informational class for everything safely escapable
+# via "as_escape()"
+{
+  sub _is_escapable {
+    UNIVERSAL::isa($_[0], 'YATT::Lite::Util::escapable');
+  }
+  package
+    YATT::Lite::Util::escapable;
+}
+
 {
   package
     YATT::Lite::Util::named_attr;
+  BEGIN {our @ISA = ('YATT::Lite::Util::escapable')};
   use overload qw("" as_string);
   sub as_string {
     shift->[-1];
@@ -709,7 +722,7 @@ sub secure_text_plain {
 
 sub psgi_error {
   my ($self, $status, $msg, @rest) = @_;
-  return [$status, [$self->secure_text_plain, @rest], [$msg]];
+  return [$status, [$self->secure_text_plain, @rest], [escape($msg)]];
 }
 
 sub ixhash {
@@ -721,7 +734,7 @@ sub ixhash {
 sub parse_nested_query {
   return {} unless defined $_[0] and $_[0] ne '';
   my ($enc) = $_[1];
-  my $params = ixhash();
+  my $params = $_[2] // ixhash();
   foreach my $p (split /[;&]/, $_[0]) {
     my ($k, $v) = map {
       s/\+/ /g;
